@@ -11,6 +11,9 @@
 /* Define to disable the high quality mixer (build only with the standart mixer) */
 /*#define NO_HQMIXER*/
 
+/* disable support for module depackers */
+/*#define NO_DEPACKERS*/
+
 /* Define if your system supports binary pipes (i.e. Unix) */
 /*#define DRV_PIPE*/
 /* Define if you want support for output to stdout */
@@ -40,7 +43,9 @@
 #define HAVE_SETENV
 
 /* Define if you have the srandom function.  */
+#if defined(__DJGPP__)
 #define HAVE_SRANDOM
+#endif
 
 /* Define if you have the <fcntl.h> header file.  */
 #define HAVE_FCNTL_H
