@@ -42,17 +42,56 @@
 
 #include <windows.h>
 
+#if defined(_MSC_VER)
 #pragma comment(lib,"winmm.lib")
+#if (_MSC_VER < 1300)
+typedef DWORD DWORD_PTR;
+#endif
+#endif
 
-#define NUMBUFFERS	6				/* number of buffers */
-#define BUFFERSIZE	120				/* buffer size in milliseconds */
+/* PF_XMMI64_INSTRUCTIONS_AVAILABLE not in all SDKs. */
+#ifndef PF_XMMI64_INSTRUCTIONS_AVAILABLE
+#define PF_XMMI64_INSTRUCTIONS_AVAILABLE 10
+#endif
 
-HWAVEOUT	hwaveout;
-WAVEHDR		header[NUMBUFFERS];
-LPSTR		buffer[NUMBUFFERS];		/* pointers to buffers */
-WORD		buffersout;				/* number of buffers playing/about to be played */
-WORD		nextbuffer;				/* next buffer to be mixed */
-ULONG		buffersize;				/* buffer size in bytes */
+#ifndef WAVE_FORMAT_IEEE_FLOAT
+#define WAVE_FORMAT_IEEE_FLOAT 0x0003
+#endif
+
+#define MINBUFFERS		2			/* min number of buffers */
+#define MAXBUFFERS		16			/* max number of buffers */
+#define MINBUFFERSIZE_MS	50			/* min buffer size in milliseconds */
+#define MAXBUFFERSIZE_MS	1000			/* max buffer size in milliseconds */
+
+#define DEFAULT_NUMBUFFERS	2
+#define DEFAULT_BUFFERSIZE_MS	120
+
+static HWAVEOUT	hwaveout;
+static WAVEHDR	header[MAXBUFFERS];
+static HPSTR	buffer[MAXBUFFERS];		/* pointers to buffers */
+static WORD	buffersout;				/* number of buffers playing/about to be played */
+static WORD	nextbuffer;				/* next buffer to be mixed */
+static ULONG	buffersize;				/* buffer size in bytes */
+static ULONG	buffersize_ms = DEFAULT_BUFFERSIZE_MS;	/* configured buffer size in milliseconds */
+static ULONG	numbuffers = DEFAULT_NUMBUFFERS;	/* configured number of buffers */
+
+static void WIN_CommandLine(const CHAR *cmdline)
+{
+	CHAR *ptr;
+
+	if((ptr = MD_GetAtom("buffer",cmdline,0)) != NULL) {
+		buffersize_ms = atoi(ptr);
+		if(buffersize_ms < MINBUFFERSIZE_MS || buffersize_ms > MAXBUFFERSIZE_MS)
+			buffersize_ms = DEFAULT_BUFFERSIZE_MS;
+		MikMod_free(ptr);
+	}
+	if((ptr = MD_GetAtom("count",cmdline,0)) != NULL) {
+		numbuffers = atoi(ptr);
+		if(numbuffers < MINBUFFERS || numbuffers > MAXBUFFERS)
+			numbuffers = DEFAULT_NUMBUFFERS;
+		MikMod_free(ptr);
+	}
+}
 
 /* converts Windows error to libmikmod error */
 static int WIN_GetError(MMRESULT mmr)
@@ -80,28 +119,29 @@ static BOOL WIN_IsThere(void)
 	return waveOutGetNumDevs()>0?1:0;
 }
 
-static void CALLBACK WIN_CallBack(HWAVEOUT hwo,UINT uMsg,DWORD dwInstance,DWORD dwParam1,DWORD dwParam2)
+static void CALLBACK WIN_CallBack(HWAVEOUT hwo,UINT uMsg,DWORD_PTR dwInstance,DWORD_PTR dwParam1,DWORD_PTR dwParam2)
 {
 	if (uMsg==WOM_DONE) --buffersout;
 }
-  
-static BOOL WIN_Init(void)
+
+static int WIN_Init(void)
 {
 	WAVEFORMATEX	wfe;
 	WORD			samplesize;
 	MMRESULT		mmr;
-	int				n;
+	int		n;
 
 	samplesize=1;
 	if (md_mode&DMODE_STEREO) samplesize<<=1;
-	if (md_mode&DMODE_16BITS) samplesize<<=1;
+	if (md_mode&DMODE_FLOAT) samplesize<<=2;
+	else if (md_mode&DMODE_16BITS) samplesize<<=1;
 
-	wfe.wFormatTag=WAVE_FORMAT_PCM;
+	wfe.wFormatTag=(md_mode&DMODE_FLOAT)? WAVE_FORMAT_IEEE_FLOAT : WAVE_FORMAT_PCM;
 	wfe.nChannels=md_mode&DMODE_STEREO?2:1;
 	wfe.nSamplesPerSec=md_mixfreq;
 	wfe.nAvgBytesPerSec=md_mixfreq*samplesize;
 	wfe.nBlockAlign=samplesize;
-	wfe.wBitsPerSample=md_mode&DMODE_16BITS?16:8;
+	wfe.wBitsPerSample=(md_mode&DMODE_FLOAT)?32:(md_mode&DMODE_16BITS)?16:8;
 	wfe.cbSize=sizeof(wfe);
 
 	mmr=waveOutOpen(&hwaveout,WAVE_MAPPER,&wfe,(DWORD_PTR)WIN_CallBack,0,CALLBACK_FUNCTION);
@@ -110,10 +150,10 @@ static BOOL WIN_Init(void)
 		return 1;
 	}
 
-	buffersize=md_mixfreq*samplesize*BUFFERSIZE/1000;
+	buffersize=md_mixfreq*samplesize*buffersize_ms/1000;
 
-	for (n=0;n<NUMBUFFERS;n++) {
-		buffer[n]=MikMod_malloc(buffersize);
+	for (n=0;n<numbuffers;n++) {
+		buffer[n]=(HPSTR)MikMod_malloc(buffersize);
 		header[n].lpData=buffer[n];
 		header[n].dwBufferLength=buffersize;
 		mmr=waveOutPrepareHeader(hwaveout,&header[n],sizeof(WAVEHDR));
@@ -127,11 +167,12 @@ static BOOL WIN_Init(void)
 	}
 
 	md_mode|=DMODE_SOFT_MUSIC|DMODE_SOFT_SNDFX;
-	// This test only works on Windows XP or later
-	if (IsProcessorFeaturePresent(PF_XMMI64_INSTRUCTIONS_AVAILABLE))
-	{
+#if defined HAVE_SSE2
+	/* This test only works on Windows XP or later */
+	if (IsProcessorFeaturePresent(PF_XMMI64_INSTRUCTIONS_AVAILABLE)) {
 		md_mode|=DMODE_SIMDMIXER;
 	}
+#endif
 	buffersout=nextbuffer=0;
 	return VC_Init();
 }
@@ -142,10 +183,11 @@ static void WIN_Exit(void)
 
 	VC_Exit();
 	if (hwaveout) {
-		for (n=0;n<NUMBUFFERS;n++) {
+		for (n=0;n<numbuffers;n++) {
 			if (header[n].dwFlags&WHDR_PREPARED)
 				waveOutUnprepareHeader(hwaveout,&header[n],sizeof(WAVEHDR));
 			MikMod_free(buffer[n]);
+			buffer[n] = NULL;
 		}
 		while (waveOutClose(hwaveout)==WAVERR_STILLPLAYING) Sleep(10);
 		hwaveout=NULL;
@@ -156,12 +198,12 @@ static void WIN_Update(void)
 {
 	ULONG done;
 
-	while (buffersout<NUMBUFFERS) {
-		done=VC_WriteBytes(buffer[nextbuffer],buffersize);
+	while (buffersout<numbuffers) {
+		done=VC_WriteBytes((SBYTE*)buffer[nextbuffer],buffersize);
 		if (!done) break;
 		header[nextbuffer].dwBufferLength=done;
 		waveOutWrite(hwaveout,&header[nextbuffer],sizeof(WAVEHDR));
-		if (++nextbuffer>=NUMBUFFERS) nextbuffer%=NUMBUFFERS;
+		if (++nextbuffer>=numbuffers) nextbuffer%=numbuffers;
 		++buffersout;
 	}
 }
@@ -175,11 +217,12 @@ static void WIN_PlayStop(void)
 MIKMODAPI MDRIVER drv_win={
 	NULL,
 	"Windows waveform-audio",
-	"Windows waveform-audio driver v0.1",
+	"Windows waveform-audio driver v0.2",
 	0,255,
 	"winmm",
-	NULL,
-	NULL,
+	"buffer:r:50,1000,120:Audio buffer size in milliseconds\n"
+	"count:r:2,16,2:Audio buffer count\n",
+	WIN_CommandLine,
 	WIN_IsThere,
 	VC_SampleLoad,
 	VC_SampleUnload,
