@@ -19,19 +19,10 @@
 */
 
 /*==============================================================================
-
-  $Id$
-
   Driver for Windows Sound System under DOS
-
+  Written by Andrew Zabolotny <bit@eltech.ru>
 ==============================================================================*/
 
-/*
-
-	Written by Andrew Zabolotny <bit@eltech.ru>
-
-*/
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -53,15 +44,15 @@ static void WSS_CommandLine(const CHAR *cmdline)
 {
 	char *ptr, *end;
 
-	if ((ptr=MD_GetAtom("port",cmdline,0))) {
+	if ((ptr=MD_GetAtom("port",cmdline,0)) != NULL) {
 		wss.port = strtol(ptr, &end, 16);
 		MikMod_free(ptr);
 	}
-	if ((ptr=MD_GetAtom("irq",cmdline,0))) {
+	if ((ptr=MD_GetAtom("irq",cmdline,0)) != NULL) {
 		wss.irq = strtol(ptr, &end, 10);
 		MikMod_free(ptr);
 	}
-	if ((ptr=MD_GetAtom("dma",cmdline,0))) {
+	if ((ptr=MD_GetAtom("dma",cmdline,0)) != NULL) {
 		wss.dma = strtol(ptr, &end, 10);
 		MikMod_free(ptr);
 	}
@@ -213,5 +204,3 @@ MDRIVER drv_wss =
 MISSING(drv_wss);
 
 #endif
-
-/* ex:set ts=4: */
