@@ -114,6 +114,7 @@ static BOOL STX_Test(void)
 	UBYTE id[8];
 	int t;
 
+	memset(id,0,8);
 	_mm_fseek(modreader,0x3C,SEEK_SET);
 	if(!_mm_read_UBYTES(id,4,modreader)) return 0;
 	if(memcmp(id,"SCRM",4)) return 0;
@@ -301,6 +302,11 @@ static BOOL STX_Load(BOOL curious)
 		_mm_errno = MMERR_LOADING_HEADER;
 		return 0;
 	}
+	if(mh->ordnum > 256 || !mh->insnum || mh->insnum > 256 ||
+	   mh->patnum > 254 || !mh->patnum) {
+		_mm_errno = MMERR_NOT_A_MODULE;
+		return 0;
+	}
 
 	/* set module variables */
 	of.songname    = DupStr(mh->songname,20,1);
@@ -393,7 +399,7 @@ static BOOL STX_Load(BOOL curious)
 		q->loopstart  = s.loopbeg;
 		q->loopend    = s.loopend;
 		q->volume     = s.volume;
-		q->seekpos    = (((long)s.memsegh)<<16|s.memsegl)<<4;
+		q->seekpos    = (((ULONG)s.memsegh)<<16|s.memsegl)<<4;
 		q->flags     |= SF_SIGNED;
 
 		if(s.flags&1) q->flags |= SF_LOOP;
