@@ -120,6 +120,9 @@ static void _mm_registeralldrivers(void)
 #ifdef DRV_OSLES
 	_mm_registerdriver(&drv_osles);
 #endif
+#ifdef DRV_N64
+	_mm_registerdriver(&drv_n64);
+#endif
 
 	/* dos drivers - wss first, since some cards emulate sb */
 #ifdef DRV_WSS
