@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
- 
+
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
- 
+
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -32,7 +32,7 @@
 
 #include "mikmod_internals.h"
 
-void MikMod_RegisterAllLoaders_internal(void)
+static void MikMod_RegisterAllLoaders_internal(void)
 {
 	_mm_registerloader(&load_669);
 	_mm_registerloader(&load_amf);
@@ -40,7 +40,7 @@ void MikMod_RegisterAllLoaders_internal(void)
 	_mm_registerloader(&load_dsm);
 	_mm_registerloader(&load_far);
 	_mm_registerloader(&load_gdm);
-	_mm_registerloader(&load_gt2);
+/*	_mm_registerloader(&load_gt2);*/ /* load_gt2 isn't complete */
 	_mm_registerloader(&load_it);
 	_mm_registerloader(&load_imf);
 	_mm_registerloader(&load_mod);
@@ -51,13 +51,14 @@ void MikMod_RegisterAllLoaders_internal(void)
 	_mm_registerloader(&load_stm);
 	_mm_registerloader(&load_stx);
 	_mm_registerloader(&load_ult);
+	_mm_registerloader(&load_umx);
 	_mm_registerloader(&load_uni);
 	_mm_registerloader(&load_xm);
 
 	_mm_registerloader(&load_m15);
 }
 
-void MikMod_RegisterAllLoaders(void)
+MIKMODAPI void MikMod_RegisterAllLoaders(void)
 {
 	MUTEX_LOCK(lists);
 	MikMod_RegisterAllLoaders_internal();
